% PSD_LOMB computes Lomb-Scargle Periodogram.
% PSD estimation for unevenly spaced data y(t), at arbitrary frequencies
% specified by 'freq'.

function power = psd_lomb(t,y,freq)

    % Set constants
    nfreq = length(freq);
    power = zeros(nfreq,1);
    f4pi = freq*4.*pi;pi2=pi*2.;
    n = length(y);
    var = cov(y);%variance
    
    % Subtract mean
    yn = y - mean(y);
    
    % Do one Lomb loop ( the t loop is vectorised)
    for fi = 1:nfreq
        
        sinsum = sum(sin(f4pi(fi)*t));
        cossum = sum(cos(f4pi(fi)*t));
        tau = atan2(sinsum,cossum);
        argu = pi2*freq(fi)*(t-tau);
        cosarg = cos(argu);
        cfi = sum(yn.*cosarg);
        cosnorm = sum(cosarg.*cosarg);
        sinarg = sin(argu);
        sfi = sum(yn.*sinarg);
        sinnorm = sum(sinarg.*sinarg);
        %power(fi) = (cfi*cfi/cosnorm+sfi*sfi/sinnorm)/(2*var);
        power(fi) = (cfi*cfi/cosnorm+sfi*sfi/sinnorm)/(2);
    end